@@ -2,39 +2,39 @@
 /* BEGIN_COMMON_COPYRIGHT_HEADER
  *
  * TOra - An Oracle Toolkit for DBA's and developers
- * 
+ *
  * Shared/mixed copyright is held throughout files in this product
- * 
+ *
  * Portions Copyright (C) 2000-2001 Underscore AB
  * Portions Copyright (C) 2003-2005 Quest Software, Inc.
  * Portions Copyright (C) 2004-2009 Numerous Other Contributors
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation;  only version 2 of
  * the License is valid for this program.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- * 
+ *
  *      As a special exception, you have permission to link this program
  *      with the Oracle Client libraries and distribute executables, as long
  *      as you follow the requirements of the GNU GPL in regard to all of the
  *      software in the executable aside from Oracle client libraries.
- * 
+ *
  *      Specifically you are not permitted to link this program with the
  *      Qt/UNIX, Qt/Windows or Qt Non Commercial products of TrollTech.
  *      And you are not permitted to distribute binaries compiled against
- *      these libraries. 
- * 
+ *      these libraries.
+ *
  *      You may link this product with any GPL'd Qt library.
- * 
+ *
  * All trademarks belong to their respective owners.
  *
  * END_COMMON_COPYRIGHT_HEADER */
@@ -46,6 +46,7 @@
 #include "totool.h"
 #include "tohighlightedtext.h"
 #include "tosqlparse.h"
+#include "toplsqltext.h"
 
 class QMdiSubWindow;
 class QTreeView;
@@ -54,7 +55,7 @@ class toConnection;
 class toPLSQLWidget;
 class toTreeWidget;
 class toTreeWidgetItem;
-
+class toPLSQLText;
 
 /*! \brief Advanced PL/SQL Editor. It's based on
 Debugger code. But it can compile PL/SQL unit in all cases.
@@ -67,6 +68,8 @@ class toPLSQLEditor : public toToolWidget
 {
     Q_OBJECT;
 
+    toConnection * conn;
+
     // Toolbar
     void createActions(void);
 
@@ -79,6 +82,7 @@ class toPLSQLEditor : public toToolWidget
     QAction   *closeAllAct;
     QAction   *nextErrorAct;
     QAction   *previousErrorAct;
+    QAction   *checkCodeAct;
 
     // Extra menu
     QMenu *ToolMenu;
@@ -104,6 +108,10 @@ class toPLSQLEditor : public toToolWidget
                     bool current = false);
 
     void closeEditor(toPLSQLWidget* &editor);
+    void parseResults(const QString buf, QMultiMap<int, QString> &res);
+
+    // checks if given object has any source code
+    bool hasCode(const QString &schema, const QString &type, const QString &name);
 
 protected:
     virtual void closeEvent(QCloseEvent *);
@@ -115,6 +123,7 @@ public:
 
     toPLSQLWidget *currentEditor(void);
     QString currentSchema(void);
+    toPLSQLText *getAnotherPart(QString &pSchema, QString &pObject, QString &pType);
 
 public slots:
     void compile(void);
@@ -130,6 +139,7 @@ public slots:
     void windowActivated(QMdiSubWindow *w);
     void closeEditor(void);
     void closeAllEditor(void);
+    void checkCode(void);
 };
 
 #endif
