@@ -360,7 +360,7 @@ static QString QueryParam(const QString &in, toQList &params, std::list<QString>
                         break;
                     }
                     if (cpar == params.end())
-                        throw QString::fromLatin1("Not all bind variables supplied");
+                        throw toConnection::exception(QString::fromLatin1("Not all bind variables supplied"), i);
                     if ((*cpar).isNull())
                     {
                         str = QString::fromLatin1("NULL");
@@ -1087,30 +1087,30 @@ class qSqlSetting : public QWidget, public toSettingTab
 
     static QString fromQSqlName(const QString &driv)
     {
-        if (driv == QString::fromLatin1("QMYSQL3"))
+        if (driv == QString::fromLatin1("QMYSQL"))
             return "MySQL";
-        else if (driv == QString::fromLatin1("QPSQL7"))
+        else if (driv == QString::fromLatin1("QPSQL"))
             return "PostgreSQL";
         else if (driv == QString::fromLatin1("QTDS"))
             return "Microsoft SQL/TDS";
-        else if (driv == QString::fromLatin1("QSAPDB7"))
+        else if (driv == QString::fromLatin1("QSAPDB"))
             return "SapDB";
-        else if (driv == QString::fromLatin1("QODBC3"))
+        else if (driv == QString::fromLatin1("QODBC"))
             return "ODBC";
         return "";
     }
     static QString toQSqlName(const QString &driv)
     {
         if (driv == "MySQL")
-            return QString::fromLatin1("QMYSQL3");
+            return QString::fromLatin1("QMYSQL");
         else if (driv == "PostgreSQL")
-            return QString::fromLatin1("QPSQL7");
+            return QString::fromLatin1("QPSQL");
         else if (driv == "Microsoft SQL/TDS")
             return QString::fromLatin1("QTDS");
         else if (driv == "SapDB")
-            return QString::fromLatin1("QSAPDB7");
+            return QString::fromLatin1("QSAPDB");
         else if (driv == "ODBC")
-            return QString::fromLatin1("QODBC3");
+            return QString::fromLatin1("QODBC");
         return QString::null;
     }
 
@@ -1406,11 +1406,16 @@ class qSqlSetting : public QWidget, public toSettingTab
         }
         virtual int rowsProcessed(void)
         {
-            LockingPtr<QSqlDatabase> ptr(Connection->Connection, Connection->Lock);
+            try {
+                LockingPtr<QSqlDatabase> ptr(Connection->Connection, Connection->Lock, true);
 
-            if (!Query)
+                if (!Query)
+                    return 0;
+                return Query->numRowsAffected();
+            }
+            catch(...) {
                 return 0;
-            return Query->numRowsAffected();
+            }
         }
         virtual int columns(void)
         {
@@ -1745,14 +1750,19 @@ class qSqlConnection : public toConnection::connectionImpl
         }
     }
 
+
     ~toQSqlProvider()
     {
+#ifdef Q_WS_MAC
+#warning "toQSqlProvider is disabled for Mac OS X due crashes. Maybe it is not a correct solution..."
+#else
         for (int i = 0;i < Drivers.count();i++)
         {
             QString t = fromQSqlName(Drivers[i]);
             if (!t.isEmpty())
                 removeProvider(t);
         }
+#endif
     }
 
     virtual QWidget *providerConfigurationTab(const QString &provider,
