@@ -2,39 +2,39 @@
 /* BEGIN_COMMON_COPYRIGHT_HEADER
  *
  * TOra - An Oracle Toolkit for DBA's and developers
- * 
+ *
  * Shared/mixed copyright is held throughout files in this product
- * 
+ *
  * Portions Copyright (C) 2000-2001 Underscore AB
  * Portions Copyright (C) 2003-2005 Quest Software, Inc.
  * Portions Copyright (C) 2004-2009 Numerous Other Contributors
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation;  only version 2 of
  * the License is valid for this program.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- * 
+ *
  *      As a special exception, you have permission to link this program
  *      with the Oracle Client libraries and distribute executables, as long
  *      as you follow the requirements of the GNU GPL in regard to all of the
  *      software in the executable aside from Oracle client libraries.
- * 
+ *
  *      Specifically you are not permitted to link this program with the
  *      Qt/UNIX, Qt/Windows or Qt Non Commercial products of TrollTech.
  *      And you are not permitted to distribute binaries compiled against
- *      these libraries. 
- * 
+ *      these libraries.
+ *
  *      You may link this product with any GPL'd Qt library.
- * 
+ *
  * All trademarks belong to their respective owners.
  *
  * END_COMMON_COPYRIGHT_HEADER */
@@ -56,7 +56,7 @@
 #include <QSize>
 #include <QFont>
 #include <QFontMetrics>
-
+#include <QProgressDialog>
 
 toResultTableViewEdit::toResultTableViewEdit(bool readable,
         bool numberColumn,
@@ -153,7 +153,7 @@ void toResultTableViewEdit::recordChange(const QModelIndex &index,
     }
 
     // don't record if not changed
-    if(newValue == row[index.column()])
+    if (newValue == row[index.column()])
         return;
 
     struct ChangeSet change;
@@ -200,7 +200,7 @@ void toResultTableViewEdit::commitDelete(ChangeSet &change, toConnection &conn)
     const toResultModel::HeaderList Headers = Model->headers();
     bool oracle = toIsOracle(conn);
 
-    QString sql = QString("DELETE FROM %1.%2 ").arg(Owner).arg(Table);
+    QString sql = QString("DELETE FROM %1.%2 ").arg(conn.quote(Owner)).arg(conn.quote(Table));
     sql += (" WHERE ");
 
     bool where = false;
@@ -259,7 +259,7 @@ void toResultTableViewEdit::commitAdd(ChangeSet &change, toConnection &conn)
 {
     const toResultModel::HeaderList Headers = Model->headers();
 
-    QString sql = QString("INSERT INTO %1.%2 (").arg(Owner).arg(Table);
+    QString sql = QString("INSERT INTO %1.%2 (").arg(conn.quote(Owner)).arg(conn.quote(Table));
 
     int num = 0;
     for (int i = 1; i < change.row.size(); i++)
@@ -319,7 +319,7 @@ void toResultTableViewEdit::commitUpdate(ChangeSet &change, toConnection &conn)
     const toResultModel::HeaderList Headers = Model->headers();
     bool oracle = toIsOracle(conn);
 
-    QString sql = QString("UPDATE %1.%2 SET ").arg(Owner).arg(Table);
+    QString sql = QString("UPDATE %1.%2 SET ").arg(conn.quote(Owner)).arg(conn.quote(Table));
     sql += conn.quote(change.columnName);
 
     if (change.newValue.isNull())
@@ -448,9 +448,20 @@ bool toResultTableViewEdit::commitChanges(bool status)
 
     toConnection &conn = connection();
 
+    QProgressDialog progress(tr("Performing changes"),
+                             tr("Cancel"),
+                             0,
+                             Changes.size(),
+                             this);
+
     bool error = false;
     for (int changeIndex = 0; changeIndex < Changes.size(); changeIndex++)
     {
+        progress.setValue(changeIndex);
+        qApp->processEvents();
+        if (progress.wasCanceled())
+            break;
+
         try
         {
             struct ChangeSet &change = Changes[changeIndex];
@@ -469,8 +480,6 @@ bool toResultTableViewEdit::commitChanges(bool status)
             default:
                 toStatusMessage(tr("Internal error."));
             }
-
-            toStatusMessage(tr("Saved"), false, false);
         }
         catch (const QString &str)
         {
@@ -480,6 +489,8 @@ bool toResultTableViewEdit::commitChanges(bool status)
         }
     }
 
+    toStatusMessage(tr("Saved %1 changes").arg(Changes.size(), 0, 10), false, false);
+
     if (error)
         refresh();
     else
@@ -517,11 +528,11 @@ void toResultTableViewEdit::revertChanges()
 
 
 void toResultTableViewEdit::handleNewRows(const QModelIndex &parent,
-                                          int start,
-                                          int end)
+        int start,
+        int end)
 {
     int col = selectionModel()->currentIndex().column();
-    if(col < 1)
+    if (col < 1)
         col = 1;
     QModelIndex index = Model->index(start - 1, col);
 
